import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import scipy.optimize as so


class Func_med:
    def __init__(self, f):
        self.f = f
    def v(self, t):
        #return self.f(t[0]) - 0.5
        return self.f(t) - 0.5


X = 1
N = 100

d_func = ss.beta(X+1, N-X+1)

t_range = np.linspace(0, 0.05)
t_v = d_func.pdf(t_range)

mean_v = (X+1) / (N+2)
mode_v = X / N
print('mean = ', mean_v)
print('mode = ', mode_v)

d_func = ss.beta(X+1, N-X+1)
func_med = Func_med(d_func.cdf)
f_med = func_med.v
median_v = so.bisect(f_med, 0.0, 1.0)

plt.title('Beta(θ|2,100)', fontsize = 16)
plt.plot([0, 0.05], [0.0, 0.0], color = 'k', ls = '-', lw = 1)
plt.plot(t_range, t_v, color = 'k', lw = 2)
plt.plot([mean_v, mean_v], [0, d_func.pdf(mean_v)], c = 'k', lw = 1.5, 
         ls = '-.', label = f'mean:{mean_v:.4f}')
plt.plot([median_v, median_v], [0, d_func.pdf(median_v)], c = 'k', lw = 1.5, 
         linestyle = (0, (5,1,2,1,2,1)), label = f'median:{median_v:.4f}')
plt.plot([mode_v, mode_v], [0, d_func.pdf(mode_v)], c = 'k', 
         ls = '--', lw = 1.5, label = f'mode:{mode_v:.4f}')
plt.xlabel('θ', fontsize = 16)
plt.xticks(fontsize = 14)
plt.yticks([])
plt.legend(fontsize = 14)
plt.tight_layout()
plt.savefig('Figure1.png')
plt.show()
