import pystan
import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import seaborn as sb
import pickle
import random
import tkinter as tk

sm = pystan.StanModel(file = 'OneVarNormal.stan')

with open('sm.pkl', 'wb') as f:
    pickle.dump(sm, f)

n_simu =  1000  

fout = open(f'ResultsCheck.txt', 'w')

Biases_mu_mean = []
Biases_mu_median = []
Biases_mu_map = []
SEBiases_mu_mean = []
SEBiases_mu_median = []
SEBiases_mu_map = []
Biases_sgm_mean = []
Biases_sgm_median = []
Biases_sgm_map = []
SEBiases_sgm_mean = []
SEBiases_sgm_median = []
SEBiases_sgm_map = []

rms_mu_mean = []
rms_mu_median = []
rms_mu_map = []
se_rms_mu_mean = []
se_rms_mu_median = []
se_rms_mu_map = []

rms_sgm_mean = []
rms_sgm_median = []
rms_sgm_map = []
se_rms_sgm_mean = []
se_rms_sgm_median = []
se_rms_sgm_map = []

n_samples = 20

mu_means = []
mu_medians = []
mu_maps = []
sgm_means = []
sgm_medians = []
sgm_maps = []

root = tk.Tk()
root.attributes('-topmost', True)
cnvs = tk.Canvas(root, bg = '#ffffff', width = 400, height = 200)
cnvs.pack()
cnvs.create_text(200, 100, text = 'Simulation started.',font = ('Gothic', 48), tags = 'mytext') 
cnvs.update()

for trial in range(n_simu):
    cnvs.itemconfig('mytext', text = '{0}/{1}'.format(trial+1, n_simu))
    cnvs.update()

    X = ss.norm().rvs(size = n_samples)
    Data = {'N': len(X), 'X': X}
    with open('sm.pkl', 'rb') as f:
        sm = pickle.load(f)
    fit = sm.sampling(data = Data, control=dict(adapt_delta=0.97, max_treedepth = 30), n_jobs = 1)

    Smpl_mu = fit['mu']
    Mean_E = Smpl_mu.mean()
    mu_means.append(Mean_E)
    Med_E = np.percentile(Smpl_mu, 50)
    mu_medians.append(Med_E)
    L_mu = np.percentile(Smpl_mu, 2.5)
    U_mu = np.percentile(Smpl_mu, 97.5)
    n_points = 10000
    coord = np.linspace(L_mu, U_mu, n_points)
    est_pdf = ss.gaussian_kde(Smpl_mu).pdf(coord)
    map_idx = np.argmax(est_pdf)
    MAP_E = coord[map_idx]
    mu_maps.append(MAP_E)

    Smpl_sgm = fit['sgm']
    Mean_E = Smpl_sgm.mean()
    sgm_means.append(Mean_E)
    Med_E = np.percentile(Smpl_sgm, 50)
    sgm_medians.append(Med_E)
    L_sgm = 0.0 # Smpl_sgm.min()
    U_sgm = np.percentile(Smpl_sgm, 99)
    n_points = 10000
    coord = np.linspace(L_sgm, U_sgm, n_points)
    est_pdf = ss.gaussian_kde(Smpl_sgm).pdf(coord)
    map_idx = np.argmax(est_pdf)
    MAP_E = coord[map_idx]
    sgm_maps.append(MAP_E)

root.destroy()

mu_means = np.array(mu_means)
mu_medians = np.array(mu_medians)
mu_maps = np.array(mu_maps)
mu_mean_bias = mu_means.mean() - 0.0
mu_median_bias = mu_medians.mean() - 0.0
mu_map_bias = mu_maps.mean() - 0
mu_mean_se = ss.sem(mu_means)
mu_median_se = ss.sem(mu_medians)
mu_map_se = ss.sem(mu_maps)

Biases_mu_mean.append(mu_mean_bias)
Biases_mu_median.append(mu_median_bias)
Biases_mu_map.append(mu_map_bias)
SEBiases_mu_mean.append(mu_mean_se)
SEBiases_mu_median.append(mu_median_se)
SEBiases_mu_map.append(mu_map_se)

mss_mu_means = ((mu_means - 0.0) ** 2).mean()
rms_mu_mean.append(mss_mu_means ** 0.5)
t_sem = ss.sem((mu_means - 0.0) ** 2)
se_rms_mu_mean.append(t_sem ** 0.5)

mss_mu_medians = ((mu_medians - 0.0) ** 2).mean()
rms_mu_median.append(mss_mu_medians ** 0.5)
t_sem = ss.sem((mu_medians - 0.0) ** 2)
se_rms_mu_median.append(t_sem ** 0.5)

mss_mu_maps = ((mu_maps - 0.0) ** 2).mean()
rms_mu_map.append(mss_mu_maps ** 0.5)
t_sem = ss.sem((mu_maps - 0.0) ** 2)
se_rms_mu_map.append(t_sem ** 0.5)

sgm_means = np.array(sgm_means)
sgm_medians = np.array(sgm_medians)
sgm_maps = np.array(sgm_maps)
sgm_mean_bias = sgm_means.mean() - 1.0
sgm_median_bias = sgm_medians.mean() - 1.0
sgm_map_bias = sgm_maps.mean() - 1.0
sgm_mean_se = ss.sem(sgm_means)
sgm_median_se = ss.sem(sgm_medians)
sgm_map_se = ss.sem(sgm_maps)

Biases_sgm_mean.append(sgm_mean_bias)
Biases_sgm_median.append(sgm_median_bias)
Biases_sgm_map.append(sgm_map_bias)
SEBiases_sgm_mean.append(sgm_mean_se)
SEBiases_sgm_median.append(sgm_median_se)
SEBiases_sgm_map.append(sgm_map_se)

mss_sgm_means = ((sgm_means - 0.0) ** 2).mean()
rms_sgm_mean.append(mss_sgm_means ** 0.5)
t_sem = ss.sem((sgm_means - 0.0) ** 2)
se_rms_sgm_mean.append(t_sem ** 0.5)

mss_sgm_medians = ((sgm_medians - 0.0) ** 2).mean()
rms_sgm_median.append(mss_sgm_medians ** 0.5)
t_sem = ss.sem((sgm_medians - 0.0) ** 2)
se_rms_sgm_median.append(t_sem ** 0.5)

mss_sgm_maps = ((sgm_maps - 0.0) ** 2).mean()
rms_sgm_map.append(mss_sgm_maps ** 0.5)
t_sem = ss.sem((sgm_maps - 0.0) ** 2)
se_rms_sgm_map.append(t_sem ** 0.5)

fout.write('*/n_samples\n')
fout.write(f'  {n_samples}')
fout.write('\n')
fout.write('*/Biases_mu_mean\n')
for v in Biases_mu_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/Biases_mu_median\n')
for v in Biases_mu_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/Biases_mu_map\n')
for v in Biases_mu_map:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SEBiases_mu_mean\n')
for v in SEBiases_mu_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SEBiases_mu_median\n')
for v in SEBiases_mu_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SEBiases_mu_map\n')
for v in SEBiases_mu_map:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/Biases_sgm_mean\n')
for v in Biases_sgm_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/Biases_sgm_median\n')
for v in Biases_sgm_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/Biases_sgm_map\n')
for v in Biases_sgm_map:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SEBiases_sgm_mean\n')
for v in SEBiases_sgm_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SEBiases_sgm_median\n')
for v in SEBiases_sgm_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SEBiases_sgm_map\n')
for v in SEBiases_sgm_map:
    fout.write(f' {v}')
fout.write('\n')

fout.write('*/rms_mu_mean\n')
for v in rms_mu_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/rms_mu_median\n')
for v in rms_mu_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/rms_mu_map\n')
for v in rms_mu_map:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/se_rms_mu_mean\n')
for v in se_rms_mu_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/se_rms_mu_median\n')
for v in se_rms_mu_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/se_rms_mu_map\n')
for v in se_rms_mu_map:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/rms_sgm_mean\n')
for v in rms_sgm_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/rms_sgm_median\n')
for v in rms_sgm_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/rms_sgm_map\n')
for v in rms_sgm_map:
    fout.write(f' {v}')
fout.write('\n')
fout.write('(*/se_rms_sgm_mean\n')
for v in se_rms_sgm_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/se_rms_sgm_median\n')
for v in se_rms_sgm_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/se_rms_sgm_map\n')
for v in se_rms_sgm_map:
    fout.write(f' {v}')
fout.write('\n')

fout.close()
    
