import pystan
import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import seaborn as sb
import pickle
import random
import tkinter as tk

sm = pystan.StanModel(file = 'TwoVarNormal.stan')

with open('sm.pkl', 'wb') as f:
    pickle.dump(sm, f)

n_samples = 50
n_simu =  200
rhos = [-0.9, -0.6, -0.3, 0.0, 0.3, 0.6, 0.9]


bias_mean = []
bias_median = []
bias_MAP = []
SE_bias_mean = []
SE_bias_median = []
SE_bias_MAP = []
RMSE_mean = []
RMSE_median = []
RMSE_MAP = []
SE_RMSE_mean = []
SE_RMSE_median = []
SE_RMSE_MAP = []


root = tk.Tk()
root.attributes('-topmost', True)
cnvs = tk.Canvas(root, bg = '#ffffff', width = 400, height = 300)
cnvs.pack()
cnvs.create_text(200, 150, text = 'Simulation started.',font = ('Gothic', 32), tags = 'mytext') 
cnvs.update()

for rho_true in rhos:
    #print('rho =', rho_true)    
    s_bias_mean = 0.0
    s_bias_median = 0.0
    s_bias_MAP = 0.0
    se_s_bias_mean = []
    se_s_bias_median = []
    se_s_bias_MAP = []
    ss_mean = 0.0
    ss_median = 0.0
    ss_MAP = 0.0
    se_ss_mean = []
    se_ss_median = []
    se_ss_MAP = []
    Means = []
    Medians = []
    MAPs = []
    for trial in range(n_simu):
        
        cnvs.itemconfig('mytext', text = f'rho = {rho_true}\ntrial = {trial+1}/{n_simu}')
        cnvs.update()
        
        X = 0
        u = np.random.normal(size = n_samples)
        v = np.random.normal(size = n_samples)
        random.shuffle(u)
        random.shuffle(v)
        t = u * ((1.0 - (rho_true**2))**0.5) + v * rho_true
        X = np.vstack([t, v]).T
        
        Data = {'N': len(X), 'X': X}
        sm = 0
        with open('sm.pkl', 'rb') as f:
            sm = pickle.load(f)
        fit = 0
        fit = sm.sampling(data = Data,  iter = 4000, n_jobs = 1)  

        Smpl_rho = fit['rho']
        Mean_E = Smpl_rho.mean()
        Med_E = np.percentile(Smpl_rho, 50)
        coord = np.linspace(-1.0, 1.0, 20000)
        est_pdf = ss.gaussian_kde(Smpl_rho).pdf(coord)
        map_idx = np.argmax(est_pdf)
        MAP_E = coord[map_idx]
        
        Means.append(Mean_E)
        Medians.append(Med_E)
        MAPs.append(MAP_E)

        s_bias_mean += Mean_E - rho_true
        s_bias_median += Med_E - rho_true
        s_bias_MAP += MAP_E - rho_true
        se_s_bias_mean.append(Mean_E - rho_true)
        se_s_bias_median.append(Med_E - rho_true)
        se_s_bias_MAP.append(MAP_E - rho_true)
        ss_mean += (Mean_E - rho_true) ** 2
        ss_median += (Med_E - rho_true) ** 2
        ss_MAP += (MAP_E - rho_true) ** 2
        se_ss_mean.append((Mean_E - rho_true) ** 2)
        se_ss_median.append((Med_E - rho_true) ** 2)
        se_ss_MAP.append((MAP_E - rho_true) ** 2)
        
    bias_mean.append(s_bias_mean / n_simu)
    bias_median.append(s_bias_median / n_simu)
    bias_MAP.append(s_bias_MAP / n_simu)
    SE_bias_mean.append(ss.sem(se_s_bias_mean))
    SE_bias_median.append(ss.sem(se_s_bias_median))
    SE_bias_MAP.append(ss.sem(se_s_bias_MAP))
    RMSE_mean.append((ss_mean / n_simu) ** 0.5)
    RMSE_median.append((ss_median / n_simu) ** 0.5)
    RMSE_MAP.append((ss_MAP / n_simu) ** 0.5)
    SE_RMSE_mean.append(ss.sem(se_ss_mean) ** 0.5)
    SE_RMSE_median.append(ss.sem(se_ss_median) ** 0.5)
    SE_RMSE_MAP.append(ss.sem(se_ss_MAP) ** 0.5)

root.destroy()

fout = open(f'ResultsN{n_samples}.txt', 'w')

print()
print('bias_mean =', bias_mean)
print('bias_edian =', bias_median)
print('bias_MAP =', bias_MAP)
print('RMSE_mean =', RMSE_mean)
print('RMSE_median =', RMSE_median)
print('RMSE_MAP =', RMSE_MAP)

fout.write('*/bias_mean\n')
for v in bias_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SE_bias_mean\n')
for v in SE_bias_mean:
    fout.write(f' {v}')
fout.write('\n')

fout.write('*/bias_median\n')
for v in bias_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SE_bias_median\n')
for v in SE_bias_median:
    fout.write(f' {v}')
fout.write('\n')

fout.write('*/bias_MAP\n')
for v in bias_MAP:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SE_bias_MAP\n')
for v in SE_bias_MAP:
    fout.write(f' {v}')
fout.write('\n')

fout.write('*/RMSE_mean\n')
for v in RMSE_mean:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SE_RMSE_mean\n')
for v in SE_RMSE_mean:
    fout.write(f' {v}')
fout.write('\n')

fout.write('*/RMSE_median\n')
for v in RMSE_median:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SE_RMSE_median\n')
for v in SE_RMSE_median:
    fout.write(f' {v}')
fout.write('\n')

fout.write('*/RMSE_MAP\n')
for v in RMSE_MAP:
    fout.write(f' {v}')
fout.write('\n')
fout.write('*/SE_RMSE_MAP\n')
for v in SE_RMSE_MAP:
    fout.write(f' {v}')
fout.write('\n')

fout.close()

