data {
    int N;
    vector[2] X[N];
}
parameters {
    real<lower = -1.0, upper = 1.0> rho;
    real mu1;
    real mu2;
    real<lower = 0.0> sgm1;
    real<lower = 0.0> sgm2;
}
transformed parameters {
    vector[2] mu;
    cov_matrix[2] cov;
    mu[1] = mu1;
    mu[2] = mu2;
    cov[1][1] = sgm1 * sgm1;
    cov[1][2] = rho * sgm1 * sgm2;
    cov[2][1] = rho * sgm1 * sgm2;
    cov[2][2] = sgm2 * sgm2;
}
model {
    rho ~ uniform(-1.0, 1.0);
    mu1 ~ normal(0.0, 1000.0);
    mu2 ~ normal(0.0, 1000.0);
    sgm1 ~ exponential(0.001);
    sgm2 ~ exponential(0.001);
    for (i in 1:N) {
        X[i] ~ multi_normal(mu, cov);
    }
}